﻿Option Strict On
Imports System.Reflection

'Assumes zero value element always has entry 'Unknown'
Public Class Enum_OnVBArticles
    'Valid data types are:
    '   Byte, Short, Integer, Long, 
    '   UByte, UShort, UInteger, ULong
    '   Default data type is Integer and is best

    <DisplayName("On VB Articles")>
    Public Enum OnVBArticles
        Unknown = 0

        <DisplayName("Win 32 API Calls")>
        <PublicationDate("4/20/2010")>
        Win32APICalls = 20

        <DisplayName("Word As Report Writer For Silverlight 4")>
        <PublicationDate("5/24/2010")>
        WordAsReportWriterForSilverlight4

        <DisplayName("What's New In Visual Basic")>
        <PublicationDate("06/01/2010")>
        WhatsNewInVisualBasic

        <DisplayName("ALM: Backup On Build - Part 1")>
        <PublicationDate("06/15/2010")>
        ALM_BackupOnBuildPart1

        <DisplayName("ALM: Backup On Build - Part 2")>
        <PublicationDate("06/16/2010")>
        ALM_BackupOnBuildPart2

        <DisplayName("Experiencing C# From a VB Developer's Perspective")>
        <PublicationDate("06/23/2010")>
        CSharpFromVBDeveloperPerspective

        <DisplayName("When Hexadecimal Is Just Not Enough")>
        <PublicationDate("07/20/2010")>
        WhenHexadecimalIsJustNotEnough

        <DisplayName("How To Get Oriented With A New Database")>
        <PublicationDate("08/02/2010")>
        HowToGetOrientedWithANewDatabase

        <DisplayName("How to Validate Forms with ASP.Net MVC 2 Data Annotations")>
        <PublicationDate("9/01/2010")>
        HowToValidateFormsWithASPNETMVC2DataAnnotations

    End Enum

    'Function name ends in 'Entry' if returns an Enum 

    'Return the name of the Enum, for completeness
    Public Shared Function ItemName(ByVal Value As OnVBArticles) As String
        Dim Result = Value.ToString()
        Return Result
    End Function

    'Return the name of the entry for this integer value
    Public Shared Function ItemName(ByVal Value As Integer) As String
        Dim Result = [Enum].GetName(GetType(OnVBArticles), Value)
        Return Result
    End Function

    'Return the Entry for the specified integer value
    Public Shared Function Entry(ByVal Value As Integer) As OnVBArticles
        Dim Result = CType(Value, OnVBArticles)
        Return Result
    End Function

    'Return the Enum entry for the specified string
    'TryParse is case-sensitive, so use LINQ instead
    Public Shared Function Entry(ByVal Value As String) As OnVBArticles
        Dim Result As OnVBArticles = OnVBArticles.Unknown
        Dim names = GetNames()
        Dim match = (From item In names Where item.ToLower() = Value.ToLower Select item).FirstOrDefault
        If (String.IsNullOrEmpty(match) = False) Then
            [Enum].TryParse(match, Result)
        End If
        Return Result
    End Function

    'Return the enumeration increased by Addend if that enumeration value exists
    Public Shared Function AddEntry(ByVal Value As OnVBArticles, ByVal Term As Integer) As OnVBArticles
        Dim Result As OnVBArticles = OnVBArticles.Unknown
        Dim NewValue = CType(Value, Integer) + Term
        If IsValid(NewValue) Then _
            Result = CType(NewValue, OnVBArticles)
        Return Result
    End Function

    'Return the enumeration decreased by Term if that enumeration value exists
    Public Shared Function SubtractEntry(ByVal Value As OnVBArticles, ByVal Term As Integer) As OnVBArticles
        Dim Result As OnVBArticles = OnVBArticles.Unknown
        Dim NewValue = CType(Value, Integer) + (Term * -1)
        If IsValid(NewValue) Then _
            Result = CType(NewValue, OnVBArticles)
        Return Result
    End Function

    'Return True if the enum value is defined in the enumeration
    'Avoid IsDefined since it loads reflection and should be avoided if possible
    Public Shared Function IsValid(ByVal Value As OnVBArticles) As Boolean
        Dim Index = -1
        Dim values = GetValues()
        Try
            Index = Enumerable.Range(0, values.Count).Where(Function(s) values(s) = Value).First
        Catch ex As Exception
        End Try
        Dim Result = (Index >= 0)
        Return Result
    End Function

    Public Shared Function IsValid(ByVal Value As Integer) As Boolean
        Dim Result = IsValid(CType(Value, OnVBArticles))
        Return Result
    End Function

    'Returns the Index of the Value in the array of values
    Public Shared Function IndexValue(ByVal Value As OnVBArticles) As Integer
        Dim Result As Integer = -1
        Dim values = GetValues()
        Try
            Result = Enumerable.Range(0, values.Length).Where(Function(s) values(s) = Value).First
        Catch ex As Exception
            Throw New Exception(ex.Message, ex)
        End Try
        Return Result
    End Function

    'Returns the Entry of the Value in the array of values
    Public Shared Function IndexEntry(ByVal Value As Integer) As OnVBArticles
        Dim Result As OnVBArticles = OnVBArticles.Unknown
        Dim values = GetValues()
        If (Value >= 0 And Value < values.Length) Then _
            Result = CType(values(Value), OnVBArticles)
        Return Result
    End Function

    'Move to separate function to avoid issue with late binding when Option Strict On
    Public Shared Function GetNames() As String()
        Return CType([Enum].GetNames(GetType(OnVBArticles)), String())
    End Function

    'Move to separate function to avoid issue with late binding when Option Strict On
    Public Shared Function GetValues() As OnVBArticles()
        Return CType([Enum].GetValues(GetType(OnVBArticles)), OnVBArticles())
    End Function

    'Returns the number of entries in the Enum
    Public Shared Function Length() As Integer
        Dim values = GetValues()
        Dim Result = values.Length
        Return Result
    End Function

    Public Shared Function BottomEntry() As OnVBArticles
        Dim values = GetValues()
        Dim Result = CType(values(0), OnVBArticles)
        Return Result
    End Function

    Public Shared Function TopEntry() As OnVBArticles
        Dim values = GetValues()
        Dim Result = CType(values(values.Length - 1), OnVBArticles)
        Return Result
    End Function

    'Return the entry for the maximum numeric value of the Enum
    Public Shared Function MaximumEntry() As OnVBArticles
        Dim values = GetValues()
        Dim maxValue = (From item In values Select item).Max()
        Dim Result = (CType(maxValue, OnVBArticles))
        Return Result
    End Function

    'Return the maximum numeric value for the Enum
    Public Shared Function MaximumValue() As Integer
        Dim values = GetValues()
        Dim Result = CType((From item In values Select item).Max(), Integer)
        Return Result
    End Function

    'Return the Minimum numeric value of the Enum
    Public Shared Function MinimumEntry() As OnVBArticles
        Dim values = GetValues()
        Dim minValue = (From item In values Select item).Min()
        Dim Result = CType(minValue, OnVBArticles)
        Return Result
    End Function

    'Return the Minimum numeric value for the Enum
    Public Shared Function MinimumValue() As Integer
        Dim values = GetValues()
        Dim Result = CType((From item In values Select item).Min(), Integer)
        Return Result
    End Function

    'Returns the next entry, returning 'Unknown' if exceed the list and not circular
    'Return 'Unknown' only if exceeded non-circular upper limit, otherwise skip 'Unknown'
    Public Shared Function NextEntry(ByVal Value As OnVBArticles, ByVal Circular As Boolean) As OnVBArticles
        Dim UnknownIndex = IndexValue(OnVBArticles.Unknown)
        Dim ErrorIndex = Integer.MinValue
        Dim values = GetValues()
        Dim NextIndex = IndexValue(Value) + 1
        If (NextIndex >= values.Length) Then
            If Circular Then NextIndex = 0 Else NextIndex = ErrorIndex
        End If

        'Handle the case that the next entry happens to be 'Unknown'
        If (NextIndex = ErrorIndex) Then
            NextIndex = OnVBArticles.Unknown
        ElseIf (NextIndex = UnknownIndex) Then
            NextIndex += 1
        End If

        Dim Result = CType(values(NextIndex), OnVBArticles)
        Return Result
    End Function

    'Returns the next entry, returning 'Unknown' if exceed the list and not circular
    Public Shared Function PrevEntry(ByVal Value As OnVBArticles, ByVal Circular As Boolean) As OnVBArticles
        Dim UnknownIndex = IndexValue(OnVBArticles.Unknown)
        Dim ErrorIndex = Integer.MinValue
        Dim values = GetValues()
        Dim PrevIndex = IndexValue(Value) - 1
        If (PrevIndex <= 0) Then
            If Circular Then PrevIndex = values.Length - 1 Else PrevIndex = ErrorIndex
        End If

        'Handle the case that the next entry happens to be 'Unknown'
        If (PrevIndex = ErrorIndex) Then
            PrevIndex = OnVBArticles.Unknown
        ElseIf (PrevIndex = UnknownIndex) Then
            PrevIndex -= 1
        End If

        Dim Result = CType(values(PrevIndex), OnVBArticles)
        Return Result
    End Function

    Public Shared Function StringAttribute(ByVal Value As OnVBArticles, ByVal AttributeName As String) As String
        Dim Result = String.Empty
        Dim Field As FieldInfo = GetType(OnVBArticles).GetField(Value.ToString, BindingFlags.GetField Or BindingFlags.Public Or BindingFlags.Static)
        Dim attribs = Field.GetCustomAttributes(True)
        Dim dn = (From item In attribs Where item.GetType().Name = AttributeName Select item).FirstOrDefault
        If (Not IsNothing(dn)) Then Result = dn.ToString
        Return Result
    End Function

    Public Shared Function DisplayName(ByVal Value As OnVBArticles) As String
        Dim Result = StringAttribute(Value, "DisplayName")
        Return Result
    End Function

    Public Shared Function DateAttribute(ByVal Value As OnVBArticles, ByVal AttributeName As String) As Nullable(Of DateTime)
        Dim Result As DateTime
        Dim Field As FieldInfo = GetType(OnVBArticles).GetField(Value.ToString, BindingFlags.GetField Or BindingFlags.Public Or BindingFlags.Static)
        Dim attribs = Field.GetCustomAttributes(True)
        Dim dn = (From item In attribs Where item.GetType().Name = AttributeName Select item).FirstOrDefault
        If (Not IsNothing(dn)) Then Result = DateTime.Parse(dn.ToString)
        Return Result
    End Function

    Public Shared Function PublicationDate(ByVal Value As OnVBArticles) As Nullable(Of DateTime)
        Dim Result As Nullable(Of DateTime) = DateAttribute(Value, "PublicationDate")
        Return Result
    End Function

    Public Shared Function ItemList(ByVal IncludeUnknown As Boolean, ByVal UseDisplayName As Boolean) As List(Of EnumItem)
        Dim Result As New List(Of EnumItem)
        Dim names = GetNames()
        Dim values = GetValues()
        For i = 0 To names.Length - 1
            If (IncludeUnknown OrElse names(i).ToLower <> "unknown") Then
                If (UseDisplayName) Then
                    Dim itemName = DisplayName(CType(values(i), OnVBArticles))
                    Result.Add(New EnumItem With {.Name = itemName, .Value = values(i)})
                Else
                    Result.Add(New EnumItem With {.Name = names(i), .Value = values(i)})
                End If
            End If
        Next
        Return Result
    End Function

End Class
